using System;
using System.Collections.Generic;
using System.Linq;
using MacGruber;
using SimpleJSON;
using UnityEngine;

namespace CheesyFX
{
    public class TouchTrigger : MonoBehaviour
    {
        public new string name;
        private string region;
        private JSONStorableFloat driver;
        private bool absolute;
        public EventTrigger enterTrigger;
        public EventTrigger exitTrigger;
        public EventTrigger timeExceededTrigger;
        public FloatTrigger timeTrigger;
        public JSONStorableFloat inputTo;
        public JSONStorableFloat inputFrom;
        public JSONStorableFloat threshold;
        private bool panelOpen;
        private JSONStorableString info = new JSONStorableString("info", "");
        private string baseInfo;
        private float _lastValue;
        // public Condition condition;

        public void Init(MVRScript script, JSONStorableFloat driver, string region, bool absoluteDefault = false, float thresholdDefault = 0f, float fromDefault = 0f, float toDefault = 1f)
        {
            this.driver = driver;
            var regionSplit = region.Split(new[] { "_" }, StringSplitOptions.None);
            name = driver.name+"Trigger"+regionSplit[1];
            baseInfo = $"{regionSplit[0]} {this.driver.name}Trigger";
            // baseInfo = $"{regionSplit[0]} {this.driver.name}Trigger{regionSplit[1]}";
            this.region = regionSplit[0];
            inputFrom = new JSONStorableFloat("Input From", fromDefault, 0f, 1f, false);
            inputTo = new JSONStorableFloat("Input To", toDefault, 0f, 1f, false);
            threshold = new JSONStorableFloat("Threshold", thresholdDefault, 0f, 1f, false);
            enterTrigger = new EventTrigger(script, "OnEnter");
            exitTrigger = new EventTrigger(script, "OnExit");
            timeExceededTrigger = new EventTrigger(script, "OnThresholdExceeded");
            timeTrigger = new FloatTrigger(script, "OnValueChanged");

            // condition = new Condition();

            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            SuperController.singleton.onAtomAddedHandlers += OnAtomAdded;
        }

        public void Trigger(float v)
        {
            if(timeTrigger.HasActions())
            {
                float delta = inputTo.val - inputFrom.val;
                if (delta > 0f)
                {
                    float remappedValue = (v - inputFrom.val) / (inputTo.val - inputFrom.val);
                    // float remappedValue = Mathf.Lerp(outputFrom.val, outputTo.val, (v - inputFrom.val) / (inputTo.val - inputFrom.val));
                    timeTrigger.Trigger(remappedValue);
                }
            }
            if(enterTrigger.HasActions() || exitTrigger.HasActions())
            {
                if (_lastValue < threshold.val)
                {
                    if (v >= threshold.val) enterTrigger.Trigger();
                }
                else if (v < threshold.val) exitTrigger.Trigger();
                _lastValue = v;
            }
        }

        private void Update()
        {
            enterTrigger.Update();
            exitTrigger.Update();
            timeTrigger.Update();
            // if (condition != null && !condition.IsMet()) return;
            var driverVal = driver.val;
            if (panelOpen) info.val = $"{baseInfo}\n{driverVal:0.00}";
            Trigger(driverVal);
        }

        public void OnDestroy()
        {
            enterTrigger.Remove();
            exitTrigger.Remove();
            timeTrigger.Remove();
            SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
            SuperController.singleton.onAtomAddedHandlers -= OnAtomAdded;
        }

        public void OnAtomRename(string oldUid, string newUid)
        {
            enterTrigger.SyncAtomNames();
            exitTrigger.SyncAtomNames();
            timeTrigger.SyncAtomNames();
            // condition.OnAtomRename();
        }

        public void OnAtomAdded(Atom atom)
        {
            // condition.AddExclusionChoice(atom);
        }

        public JSONClass Store(string subScenePrefix)
        {
            JSONClass jc = new JSONClass();
            inputFrom.Store(jc);
            inputTo.Store(jc);
            threshold.Store(jc);
            jc[enterTrigger.Name] = enterTrigger.GetJSON(subScenePrefix);
            jc[exitTrigger.Name] = exitTrigger.GetJSON(subScenePrefix);
            jc[timeTrigger.Name] = timeTrigger.GetJSON(subScenePrefix);
            // condition.excludedInfo.Store(jc);
            return jc;
        }

        public void Load(JSONClass jc, string subScenePrefix)
        {
            if(jc.HasKey(name))
            {
                JSONClass tc = jc[name].AsObject;
                inputFrom.Load(tc);
                inputTo.Load(tc);
                threshold.Load(tc);
                // condition.excludedInfo.Load(jc);
                enterTrigger.RestoreFromJSON(tc, subScenePrefix, false, true);
                exitTrigger.RestoreFromJSON(tc, subScenePrefix, false, true);
                timeTrigger.RestoreFromJSON(tc, subScenePrefix, false, true);
            }
        }

        public void OpenPanel(MVRScript script, Action back)
        {
            panelOpen = true;
            List<object> triggerUIElements = new List<object>();
            UIDynamicButton button;
            button = script.CreateButton("Return");
            button.buttonColor = new Color(0.55f, 0.90f, 1f);
            button.button.onClick.AddListener(
                () =>
                {
                    Utils.RemoveUIElements(script, triggerUIElements);
                    panelOpen = false;
                    back();
                });
            triggerUIElements.Add(button);
            UIManager.SetupInfoOneLine(triggerUIElements,"Threshold Triggers", false, script: script);
            

            // Utils.SetupInfoText(script, name, 80f, true);
            var textfield = script.CreateTextField(info, true);
            textfield.ForceHeight(100f);
            textfield.UItext.fontSize = 36;
            textfield.UItext.alignment = TextAnchor.MiddleCenter;
            triggerUIElements.Add(textfield);
            
            var spacer = script.CreateSpacer(true);
            spacer.height = 50f;
            triggerUIElements.Add(spacer);
            // UIManager.EvenLeftToRight(script, triggerUIElements);

            threshold.CreateUI(script, true, UIElements:triggerUIElements);
            button = script.CreateButton("On Threshold Exceeded", false);
            button.button.onClick.AddListener(enterTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            triggerUIElements.Add(button);
            
            button = script.CreateButton("On Threshold Undershot", false);
            button.button.onClick.AddListener(exitTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            triggerUIElements.Add(button);

            UIManager.SetupInfoOneLine(triggerUIElements,"Value Triggers", true, script: script);
            // UIManager.EvenLeftToRight(script, triggerUIElements);
            
            inputFrom.CreateUI(script, UIElements:triggerUIElements);
            inputTo.CreateUI(script, true, UIElements:triggerUIElements);
            
            button = script.CreateButton("On Value Changed", false);
            button.button.onClick.AddListener(timeTrigger.OpenPanel);
            button.buttonColor = new Color(0.45f, 1f, 0.45f);
            triggerUIElements.Add(button);
            spacer = script.CreateSpacer(true);
            spacer.height = 50f;
            triggerUIElements.Add(spacer);

            // if (condition == null) return;
            // UIManager.SetupInfoOneLine(triggerUIElements,"Conditions", true, script: script);
            // // UIManager.EvenLeftToRight();
            // button = script.CreateButton("Exclude...");
            // button.button.onClick.AddListener(() => condition.Exclude(condition.excludedChooser.val, true));
            // triggerUIElements.Add(button);
            // button = script.CreateButton("(Re)Include...");
            // button.button.onClick.AddListener(() => condition.Exclude(condition.excludedChooser.val, false));
            // triggerUIElements.Add(button);
            // condition.excludedChooser.CreateUI(script: script, rightSide: true,chooserType:2, UIElements:triggerUIElements);
            //
            // textfield = script.CreateTextField(condition.excludedInfo, true);
            // textfield.height = 200f;
            // triggerUIElements.Add(textfield);
        }
    }

    public class Condition
    {
        public JSONStorableStringChooser excludedChooser =
            new JSONStorableStringChooser("atomChooser", null, "", "... this Atom");
        public static HashSet<Atom> excludedAtoms = new HashSet<Atom>();
        public JSONStorableString excludedInfo = new JSONStorableString("excludeInfo", "<b>Excluded Atoms</b>\n");

        public Atom atom;

        public Condition()
        {
            excludedChooser.choices = SuperController.singleton.GetAtomUIDs();
        }

        public bool IsMet()
        {
            return !excludedAtoms.Contains(atom);
        }

        public void Exclude(string uid, bool val)
        {
            Atom atom = SuperController.singleton.GetAtomByUid(uid);
            if (atom == null) return;
            if (val)
            {
                if(excludedAtoms.Add(atom))
                {
                    excludedInfo.val += atom.uid + "\n";
                }
            }
            else if(excludedAtoms.Remove(atom))
            { 
                excludedInfo.val = "<b>Excluded Atoms</b>\n"+string.Join("\n", excludedAtoms.Select(x => x.uid).ToArray());
            }
        }
        
        public void ResetExcluded()
        {
            excludedAtoms.Clear();
            excludedInfo.val = "<b>Excluded Atoms</b>\n";
        }
        
        public void AddExclusionChoice(Atom atom)
        {
            if (excludedChooser.choices.Contains(atom.uid)) return;
            var choices = excludedChooser.choices;
            choices.Insert(0, atom.uid);
            excludedChooser.choices = new List<string>(choices);
        }

        public void OnAtomRename()
        {
            excludedChooser.choices = SuperController.singleton.GetAtomUIDs();
            var ignoredUids = excludedAtoms.Select(x => x.uid).ToArray();
            excludedInfo.val = "<b>Excluded Atoms</b>\n"+string.Join("\n", ignoredUids);
        }
    }
}